#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
 Socktroll server written in python
 Copyright (C) 2008-2009  Patrik Lembke

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import time
from blamlib import network

class Socktroll_Server( network.TCP_Server ):
    def on_connect( self, client ):
        print " * Connection from %s:%d" %( client.host, client.port )
        client.write( "msg server %s" % "Welcome to socktroll.py" )
        clients[ client.host + ":" + str( client.port ) ] = ""

    def on_disconnect( self, client ):
        client_id = client.host + ":" + str( client.port )

        if clients[ client_id ] != '':
            print " - %s" % clients[ client_id ]
            self.broadcast_authed( "- %s" % clients[ client_id ], client ) # send room join
        else:
            print " * Unauthed client disconnected"

        clients.__delitem__( client_id )

    def handle( self, client, data ):
        client_id = client.host + ":" + str( client.port )
        data = data.strip()

        print "[debug] nick = '%s', data = '%s'" %( clients[client_id], data )
        
        command = data.split( ' ' )[0]
        args = data.split( ' ' )[1:]
        
        # If not authed we vill only have a subset
        if clients[client_id] == "":
            self.handle_unauthed( client, command, args, client_id )
        else:
            self.handle_authed( client, command, args, client_id )

    def handle_unauthed( self, client, command, args, client_id ):
        if command == "nick" and len( args ) == 1:
            self.cmd_nick( client, client_id, args[0] )
        else:
            client.write( "illegal command" )
        
    def handle_authed( self, client, command, args, client_id ):
        if command == "nick" and len( args ) == 1:
            self.cmd_nick( client, client_id, args[0] )
        elif command == "msg":
            self.cmd_msg( client, client_id, " ".join( args ) )
        elif command == "action":
            self.cmd_action( client, client_id, " ".join( args ) )
        elif command == "names":
            self.cmd_names( client, client_id, None )
        elif command == "quit":
            self.cmd_quit( client, client_id, None )
        else:
            client.write( "illegal command" )
    
    def broadcast_authed( self, data, ignore=None ):
        """set ignore to a client to not send data to it"""
        for client in self.clients:
            if client != ignore and clients[ client.host + ":" +
                                             str( client.port ) ] != '':
                client.write( data )
        
    def cmd_nick( self, client, client_id, nick ):
        # On nick, change/auth
        rename = clients[client_id] != ""
        
        # Check if not a word object! ^\w+$
        if len( unicode( nick, "utf-8" ) ) > 20:
            client.write( "no bad" )
            return
        
        # check if not taken
        if nick in clients.values() or nick == 'server':
            client.write( "no taken" )
            return
        
        # if still here its all fine =)
        if not rename:
            client.write( "ok %s" % nick )
            self.broadcast_authed( "+ %s" % nick, client ) # send room join
            print " + %s" % nick
            
        else:
            client.write( "rename %s %s" %( clients[client_id], nick ) )
            print "rename %s %s" %( clients[client_id], nick )
            self.broadcast_authed( "rename %s %s" %( clients[client_id], nick ),
                                   client ) # send room rename
            
        clients[client_id] = nick

    def cmd_msg( self, client, client_id, text ):
        self.broadcast_authed( "msg %s %s" %( clients[client_id], text ) )

    def cmd_names( self, client, client_id, nan=None ):
        client.write( "names " + ",".join( clients.values() ) )

    def cmd_quit( self, client, client_id, nan=None ):
        self.close_client( client )

    def cmd_action( self, client, client_id, text ):
        self.broadcast_authed( "action %s %s" %( clients[client_id], text ) )

# main
if __name__ == '__main__':
    clients = dict() # ['host:ip'] = nick
    run = True
    
    print "Socktroll (python server) v0.0.1"
    print
    server = Socktroll_Server( "", 6000 )
    print " * Listning on %s:%d" %( server.host, server.port )
    
    while run:
        server.poll()
        time.sleep( 0.001 )
